;****************************************************************
;*       File Register                                          *
;****************************************************************
FLAG    EQU     60h             ; Allg Flag Bits Register
EEPROM  EQU     61h             ; Bit Speicher
ERCODE  EQU     62h             ; Error Code 
DATAI   EQU     63h             ; Daten Eingangs Registrer
DATAO   EQU     64h             ; Daten Ausgangs Registrer
TXBUF   EQU     65h             ; TX Speicher
RXBUF   EQU     66h             ; RX Speicher
COUNT   EQU     67h             ; Bit Zhler
;****************************************************************
;   ASM Makro Definitionen                                      *
;****************************************************************

Page0   macro  
    BCF     STATUS,RP0
    BCF     STATUS,RP1
    endm
Page1   macro
    BSF     STATUS,RP0
    BCF     STATUS,RP1
    endm
Page2   macro
    BCF     STATUS,RP0
    BSF     STATUS,RP1
    endm
Page3   macro
    BSF     STATUS,RP0
    BSF     STATUS,RP1
    endm            
;****************************************************************
;*                    Bit Assignments                           * 
;****************************************************************
; FLAG Bits
ERR_1   EQU     0               ; Error Flag
; EEPROM Bits
DI      EQU     7               ; EEPROM Eingang
DO      EQU     6               ; EEPROM Ausgang
; I2C  Bits
SDA     EQU     1               ; RB1, data in/out
SCL     EQU     4               ; RB4, serial clock
;****************************************************************
;   I2C Init                                                    *
;****************************************************************
_MM_I2C_Init
    clrf	FLAG		        ; Lsche Error Register
	clrf    SSPCON              ; SSP zurcksetzen (fr Timeouts)
    movlw	B'00111110'	        ; I2C 7bit Slavemode mit Master 
	movwf	SSPCON		        ; Mode einschalten
	Page1
    bsf	    TRISB,SCL	        ; Setze SCL 1
	bsf	    TRISB,SDA	        ; Setze SDA 1
	Page0
	bcf	    PIR1,3		        ; Lsche SSP-Interrupt-Flag
	return
;****************************************************************
;   I2C X-Byte-Write                                            *
;****************************************************************        
I2C_X_Byte_Write	
    movlw   .255
	movwf   COUNT
CheckAgain_X_Byte
	Page0
    decfsz  COUNT,F             ; Nach 200 Versuchen aufgeben
    goto    C_A_X_Byte          ; wenn Zhler nicht 0 dann weiter
    movlw   .5
	call    ERR                 ; Errorcode speichern
    goto    X_Byte_ERR
C_A_X_Byte
	Page1
	btfsc	SSPSTAT,4	        ;Wenn STOP bit empfangen...
	goto	Goxmit_X_Byte	    ;  OK zum Senden
	btfsc	SSPSTAT,3	        ;Wenn START bit empfangen...
	goto	CheckAgain_X_Byte	;  Warten auf STOP bit
Goxmit_X_Byte
    ; *************************Byte Schleife***************************
    ; max 8 Bytes bedeutet8 aufeinanderfolgende Speicherpltze
    ; Speicherpltze werden von $70-$7F verwendet Erster Speicherplatz
    ; ist die Gerte Adresse.
    Page0
    movlw   _SLAVE_ADDR         ; $6F Speicheradresse des ersten Byte
    movwf    FSR                ; bernehme Zeiger
	call    BSTART              ; Generiere START bit                       
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_ERR          ; Raus hier
    bcf	    FLAG,ERR_1
;*************************Byte Schleife***************************
X_BYTE_LOOP                     ; Byteschleife
    movfw   INDF                ; Lese Byte ein      
    movwf   TXBUF               ;  in TX Puffer
	call    TX                  ; ADDR-Ausgabe und detect acknowledgement
    btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_ERR          ; Raus hier
    incf    FSR,F               ; Byte Zeiger incrementieren
    decfsz  _Index,F            ; Alle Bytes gesendet?
    goto    X_BYTE_LOOP         ; Nein, nchstes
;*************************Byte Schleife***************************
	call    BSTOP               ; Generiere STOP bit	
    btfss	FLAG,ERR_1 	        ; Check for error
	goto	Checkout_X_Byte	    ; Kein error, also weiter
X_Byte_ERR
	movf	ERCODE,W	        ; Get error code
	movwf	_Errorcode          ; Errorcode Speicherm
	bcf	    FLAG,ERR_1	        ; Clear error flag
	goto    I2CERR
Checkout_X_Byte
    return
;****************************************************************
;   I2C-Bytes-Read  ohne Kombination                            *
;****************************************************************        
I2C_Bytes_Read	
	Page0
    movlw   .255
	movwf   COUNT
CheckAgain_Byte
	Page0
    decfsz  COUNT,F             ; Nach 200 Versuchen aufgeben
    goto    C_A_Byte            ; wenn Zhler nicht 0 dann weiter
    movlw   .5
	call    ERR                 ; Errorcode speichern
    goto    X_Byte_R_ERR
C_A_Byte
	Page1
	btfsc	SSPSTAT,4	        ;Wenn STOP bit empfangen...
	goto	Goxmit_Byte  	    ;  OK zum Senden
	btfsc	SSPSTAT,3	        ;Wenn START bit empfangen...
	goto	CheckAgain_Byte     ;  Warten auf STOP bit
Goxmit_Byte
    Page0
    movlw   _SLAVE_ADDR         ; $70 Speicheradresse des ersten Byte (Slaveadresse)
    movwf   FSR                 ; bernehme Zeiger
    Goto Mehr_Bytes

;****************************************************************
;   I2C X-Byte-Read                                             *
;****************************************************************        
I2C_X_Byte_Read	
    movlw   .255
	movwf   COUNT
CheckAgain_X_Byte_R
	Page0
    decfsz  COUNT,F             ; Nach 200 Versuchen aufgeben
    goto    C_A_X_Byte_R        ; wenn Zhler nicht 0 dann weiter
    movlw   .5
	call    ERR                 ; Errorcode speichern
    goto    X_Byte_R_ERR
C_A_X_Byte_R
	Page1
	btfsc	SSPSTAT,4	        ;Wenn STOP bit empfangen...
	goto	Goxmit_X_Byte_R   	;  OK zum Senden
	btfsc	SSPSTAT,3	        ;Wenn START bit empfangen...
	goto	CheckAgain_X_Byte_R	;  Warten auf STOP bit
Goxmit_X_Byte_R
    Page0
    movlw   _SLAVE_ADDR         ; $70 Speicheradresse des ersten Byte (Slaveadresse)
    movwf   FSR                 ; bernehme Zeiger
	call    BSTART              ; Generiere START bit
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_R_ERR        ; Raus hier
    movfw   INDF                ; Hole Slaveadresse
    movwf   TXBUF               ; in Sendespeicher
    Call    TX                  ; Sende Byte 
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_R_ERR        ; Raus hier 
    movfw   _Start_ADDR
    movwf   TXBUF               ; Erste Wordadresse in Sendespeicher
    Call    TX                  ; Sende Byte 
	btfsc   FLAG,ERR_1          ; Fehler ?
Mehr_Bytes:
    goto    X_Byte_R_ERR        ; Raus hier  
    call    BSTART              ; Generiere START bit frs Lesen
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_R_ERR        ; Raus hier
    movfw   INDF                ; hole Slaveadresse
    movwf   TXBUF               ; in TX Puffer
	bsf     TXBUF,0             ; Schalte READ mode (R/W = 1)
	call    TX                  ; Sende SLAVE addresse	
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_R_ERR        ; Raus hier 
;*********Leseschleife*************************************************	
X_BYTE_LOOP_R                   ; Byteschleife
	call    RX                  ; lese Byte + teste ack
	btfsc   FLAG,ERR_1          ; Fehler ?
    goto    X_Byte_R_ERR        ; Raus hier
    incf    FSR,F               ; Byte Zeiger incrementieren
	movf    RXBUF,W             ; Speicher data aus Puffer
	movwf   INDF                ; indirekt bergeben
    decfsz  _Index,F            ; Alle Bytes gelesen?
    goto    X_BYTE_LOOP_R       ; Nein, nchstes
;*********Byte Schleife************************************************
	call    BSTOP               ; Generiere STOP bit	
    btfss	FLAG,ERR_1 	        ; Check for error
	goto	Checkout_X_Byte_R   ; Kein error, also weiter
X_Byte_R_ERR
	movf	ERCODE,W	        ; Get error code
	movwf	_Errorcode          ; Errorcode Speicherm
	bcf	    FLAG,ERR_1	        ; Clear error flag
	goto    I2CERR
Checkout_X_Byte_R
    return
;****************************************************************
;*       Empfang von 8 Datenbits                                *
;****************************************************************
RX
	movlw   .8              ; 8 Datenbits
	movwf   COUNT
	clrf    RXBUF
RXLP
	rlf     RXBUF, F        ; Schiebe Daten in Puffer
	btfss   3,0
	bcf     RXBUF,0         ; carry ---> f(0)
	btfsc   3,0
	bsf     RXBUF,0
	call    BITIN   
	btfsc   EEPROM,DI
	bsf     RXBUF,0         ; Input bit =1
	decfsz  COUNT, F        ; 8 bits?
	goto    RXLP
    movf    _Index,w
	movwf   COUNT
    bsf     EEPROM,DO       ; Setze Ack Bit = 1 zum beenden
	decfsz  COUNT,F
	bcf     EEPROM,DO       ; Setze Ack Bit = 0 zum Weiterlesen
	call    BITOUT          ; Ack/Nack lesen
	retlw   0
;****************************************************************
;*       Ausgabe von 8 Daten Bits                               *
;****************************************************************
TX
	movlw   .8
	movwf   COUNT
TXLP
	bcf     EEPROM,DO       ; Lsche DatenOutBit.
	btfsc   TXBUF,7         ; Wenn bit = 0, data bit = 0
	bsf     EEPROM,DO       ; sonst data bit = 1
	call    BITOUT          ; Ausgabe Datenbit
	btfsc   FLAG,ERR_1      ; Aufhren bei Fehler
	call    ERR             ; Hinzugefgt om ERR255 zu finden
	btfsc   FLAG,ERR_1      ; "
    goto    I2CERR          ; "
    rlf     TXBUF, F        ; Bits nach links ber CarryBit schieben
	btfss   3,0             ; f(6) ---> f(7)
	bcf     TXBUF,0         ; f(7) ---> carry
	btfsc   3,0             ; carry ---> f(0)
	bsf     TXBUF,0
	decfsz  COUNT, F        ; Alle 8 Bits durch?
	goto    TXLP            ; Nein.
	call    BITIN           ; Lese Ack Bit
	movlw   3
	btfsc   EEPROM,DI       ; berprfe auf ack
	call    ERR             ; Kein Ack empfangen
	retlw   0

;****************************************************************
;*       Empfang eines einzelnen Bits                           *
;****************************************************************
BITIN
	Page1
	bsf     TRISB,SDA       ; Schalte SDA-Port auf Eingang
	Page0
	bcf     EEPROM,DI       ; Lsche DatenBit
	Page1
	bsf     TRISB,SCL       ; Setze SCL 1
	movlw   1
	Page0
CLK_W_IN
	btfsc   TRISB,SCL   	; Skip wenn SCL low ist ... Clockstretching
	                        ; evtl einen Timoutzhler hier einfgen !!!!   
    goto    BIT1
    goto    CLK_W_IN
BIT1
	btfss   PORTB,SDA       ; Teste SDA pin
	goto 	ACKOK	
	bsf     EEPROM,DI       ; DI = 1
ACKOK
	nop                     ; Delay
	nop
	nop
	nop
	Page1
    bcf     TRISB,SCL       ; Schalte SCL 0     
	Page0      ;
    retlw   0
;****************************************************************
;*       Ausgabe eines einzelnen Bit                            *
;****************************************************************
BITOUT
	btfss   EEPROM,DO       ; Ist D0 0
	goto    BIT0            ; dann bit0
	Page1
	bsf     TRISB,SDA       ; sonst Ausgabe bit 1    
	movlw   .2
	Page0
	btfsc   PORTB,SDA       ; berprfe ob Error Code 2
	goto    CLK1
	btfss   FLAG,ERR_1      ; Erhalte vorherigen Fehler Code
	movwf   ERCODE          ; Sichere Error Code
	bsf     FLAG,ERR_1      ; Setze Error Flag
	goto    CLK1           
BIT0
	Page1
	bcf     TRISB,SDA       ; Ausgabe bit 0
	nop                     ; Delay
	nop
CLK1
	Page1
	bsf     TRISB,SCL
	nop
	nop
	nop
	movlw   .1   
    Page0
CLK_W_OUT
	btfsc   PORTB,SCL       ; SCL 0?
    goto    BIT2
    goto    CLK_W_OUT       ; Ja, Warte evtl Zhler einfgen Clock-stretsching
BIT2
	nop
	nop
	nop
   	Page1
	bcf     TRISB,SCL       ; Schalte SCL 0
	Page0      
	retlw   0
;****************************************************************
;*       START Bit generierung                                  *
;****************************************************************
BSTART
	Page1
    bsf     TRISB,SDA       ; Schalte SDA 1
	bsf     TRISB,SCL       ; Schalte SCL 1
	Page0
    nop
    nop
    nop
	movlw   1               ; Fehler Code 1
	btfss   PORTB,SCL       ; 1?
	call    ERR             ; Nein, SCL bleibt 0
    Page1
	bcf     TRISB,SDA       ; SDA wird 0 whrend SCL 1
	nop                     ; Timing Einstellung, 1uS @4MHz
	nop
	nop
	bcf     TRISB,SCL       ; Starte Taktpulse
	Page0
    retlw   0
;****************************************************************
;*       STOP Bit Generierung                                   *
;****************************************************************
BSTOP
	Page1
	bcf     TRISB,SDA       ; Schalte SDA 0
	bsf     TRISB,SCL       ; Schalte SCL 1
	Page0
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw   1               ; Fehler Code 1
	btfss   PORTB,SCL       ; 1?
	call    ERR             ; Nein, SCL bleibt 0
	Page1
	bsf     TRISB,SDA       ; SDA geht von 0 auf 1 whrend SCL 1
	movlw   4               ; Fehler Code 4
	btfss   TRISB,SDA       ; 1?
    call    ERR             ; Nein, SDA bleibt 0
	Page0
	retlw   0

;****************************************************************
;*      I2C-Bus Fehlerliste                                     *
;****************************************************************
;* Nummer      Fehler                                           *
;*-------     --------------------------------------------------*
;*  1   :     SCL bleibt 0 (Bus belegt)                         *
;*  2   :     SDA bleibt 0 (Bus belegt)                         *
;*  3   :     Kein ACK (kein Handshake)                         *
;*  4   :     SDA noch 0 Master kann kein STOP bit senden       *
;****************************************************************
ERR
    Page0
    btfss   FLAG,ERR_1      ; Erhalte vorherigen Fehler Code
	movwf   ERCODE          ; Sichere error Code
	bsf     FLAG,ERR_1      ; Setze Error Flag
	retlw   0               ; 
;****************************************************************
;*       Bei Busfehler sofort den I2C-BUS Freigeben             *
;****************************************************************
I2CERR
	bsf   PORTB,SDA
	bsf   PORTB,SCL 
	Page1
	bsf   TRISB,SDA
	bsf   TRISB,SCL 
	Page0
    retlw   0
